import { Separator } from 'vscode/vscode/vs/base/common/actions';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { localize2, localize } from 'vscode/vscode/vs/nls';
import { Categories } from 'vscode/vscode/vs/platform/action/common/actionCommonCategories';
import { createAndFillInActionBarActions } from 'vscode/vscode/vs/platform/actions/browser/menuEntryActionViewItem';
import { Action2, registerAction2 } from 'vscode/vscode/vs/platform/actions/common/actions';
import { IMenuService } from 'vscode/vscode/vs/platform/actions/common/actions.service';
import { ICommandService } from 'vscode/vscode/vs/platform/commands/common/commands.service';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IContextKeyService } from 'vscode/vscode/vs/platform/contextkey/common/contextkey.service';
import { INotificationService } from 'vscode/vscode/vs/platform/notification/common/notification.service';
import { IQuickInputService } from 'vscode/vscode/vs/platform/quickinput/common/quickInput.service';
import { IUserDataProfilesService } from 'vscode/vscode/vs/platform/userDataProfile/common/userDataProfile.service';
import { PROFILES_CATEGORY, PROFILES_ENABLEMENT_CONTEXT, HAS_PROFILES_CONTEXT, MANAGE_PROFILES_ACTION_ID, ProfilesMenu } from 'vscode/vscode/vs/workbench/services/userDataProfile/common/userDataProfile';
import { IUserDataProfileManagementService, IUserDataProfileService } from 'vscode/vscode/vs/workbench/services/userDataProfile/common/userDataProfile.service';

class CreateTransientProfileAction extends Action2 {
    static { this.ID = 'workbench.profiles.actions.createTemporaryProfile'; }
    static { this.TITLE = ( localize2(8846, "Create a Temporary Profile")); }
    constructor() {
        super({
            id: CreateTransientProfileAction.ID,
            title: CreateTransientProfileAction.TITLE,
            category: PROFILES_CATEGORY,
            f1: true,
            precondition: PROFILES_ENABLEMENT_CONTEXT,
        });
    }
    async run(accessor) {
        return accessor.get(IUserDataProfileManagementService).createAndEnterTransientProfile();
    }
}
registerAction2(CreateTransientProfileAction);
class RenameProfileAction extends Action2 {
    static { this.ID = 'workbench.profiles.actions.renameProfile'; }
    constructor() {
        super({
            id: RenameProfileAction.ID,
            title: ( localize2(8847, "Rename...")),
            category: PROFILES_CATEGORY,
            f1: true,
            precondition: ( (ContextKeyExpr.and(PROFILES_ENABLEMENT_CONTEXT, HAS_PROFILES_CONTEXT))),
        });
    }
    async run(accessor, profile) {
        const quickInputService = accessor.get(IQuickInputService);
        const userDataProfileService = accessor.get(IUserDataProfileService);
        const userDataProfilesService = accessor.get(IUserDataProfilesService);
        const userDataProfileManagementService = accessor.get(IUserDataProfileManagementService);
        const notificationService = accessor.get(INotificationService);
        if (!profile) {
            profile = await this.pickProfile(quickInputService, userDataProfileService, userDataProfilesService);
        }
        if (!profile || profile.isDefault) {
            return;
        }
        const name = await quickInputService.input({
            value: profile.name,
            title: ( localize(8848, 'Rename {0}', profile.name)),
            validateInput: async (value) => {
                if (profile.name !== value && ( (userDataProfilesService.profiles.some(p => p.name === value)))) {
                    return ( localize(8849, "Profile with name {0} already exists.", value));
                }
                return undefined;
            }
        });
        if (name && name !== profile.name) {
            try {
                await userDataProfileManagementService.updateProfile(profile, { name });
            }
            catch (error) {
                notificationService.error(error);
            }
        }
    }
    async pickProfile(quickInputService, userDataProfileService, userDataProfilesService) {
        const profiles = userDataProfilesService.profiles.filter(p => !p.isDefault && !p.isTransient);
        if (!profiles.length) {
            return undefined;
        }
        const pick = await quickInputService.pick(( (profiles.map(profile => ({
            label: profile.name,
            description: profile.id === userDataProfileService.currentProfile.id ? ( localize(8850, "Current")) : undefined,
            profile
        })))), {
            title: ( localize(8851, "Rename Profile...")),
            placeHolder: ( localize(8852, "Select Profile to Rename")),
        });
        return pick?.profile;
    }
}
registerAction2(RenameProfileAction);
registerAction2(class ManageProfilesAction extends Action2 {
    constructor() {
        super({
            id: MANAGE_PROFILES_ACTION_ID,
            title: ( localize2(8853, "Manage...")),
            category: PROFILES_CATEGORY,
            precondition: ( (ContextKeyExpr.and(PROFILES_ENABLEMENT_CONTEXT, HAS_PROFILES_CONTEXT))),
        });
    }
    async run(accessor) {
        const quickInputService = accessor.get(IQuickInputService);
        const menuService = accessor.get(IMenuService);
        const contextKeyService = accessor.get(IContextKeyService);
        const commandService = accessor.get(ICommandService);
        const menu = menuService.getMenuActions(ProfilesMenu, contextKeyService);
        const actions = [];
        createAndFillInActionBarActions(menu, actions);
        if (actions.length) {
            const picks = ( (actions.map(action => {
                if (action instanceof Separator) {
                    return { type: 'separator' };
                }
                return {
                    id: action.id,
                    label: `${action.label}${action.checked ? ` $(${Codicon.check.id})` : ''}`,
                };
            })));
            const pick = await quickInputService.pick(picks, { canPickMany: false, title: PROFILES_CATEGORY.value });
            if (pick?.id) {
                await commandService.executeCommand(pick.id);
            }
        }
    }
});
registerAction2(class CleanupProfilesAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.profiles.actions.cleanupProfiles',
            title: ( localize2(8854, "Cleanup Profiles")),
            category: Categories.Developer,
            f1: true,
            precondition: PROFILES_ENABLEMENT_CONTEXT,
        });
    }
    async run(accessor) {
        return accessor.get(IUserDataProfilesService).cleanUp();
    }
});
registerAction2(class ResetWorkspacesAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.profiles.actions.resetWorkspaces',
            title: ( localize2(8855, "Reset Workspace Profiles Associations")),
            category: Categories.Developer,
            f1: true,
            precondition: PROFILES_ENABLEMENT_CONTEXT,
        });
    }
    async run(accessor) {
        const userDataProfilesService = accessor.get(IUserDataProfilesService);
        return userDataProfilesService.resetWorkspaces();
    }
});

export { RenameProfileAction };
